import React, { useState, useEffect } from 'react';
import {
  Row,
  Col,
  Card,
  Input,
  Tooltip,
  Button,
  notification,
  Spin,
  List,
  Space,
  Collapse,
  Checkbox,
  Select,
} from 'antd';
import lodash from 'lodash';
import {
  DoubleLeftOutlined,
  DoubleRightOutlined,
  DownOutlined,
  UpOutlined,
} from '@ant-design/icons';
import PageContainer from '@/components/BasePageContainer';
import {
  getWebModuleTree,
  chooseUserToStation,
  getAllGroup,
  getStationUserList,
} from '@/services/userCenter/siteManage/api';
import ListCard, {
  checkChildrenByCondition,
  getId,
} from '@/components/CheckGroup';
import styles from '@/pages/userCenter/siteManage/SiteManage.less';
import qs from 'qs';
import { appConnector } from '@/containers/App/store';
import classnames from 'classnames';
import AddModal from './AddModal';
import DelModal from './DelModal';
import EditModal from './EditModal';
import userStyles from '@/pages/userCenter/userManage/UserManage.less';

const { Search } = Input;
const { Option } = Select;

const placeholder = '请输入机构名称';
const SiteManage = props => {
  const { userMode } = props;
  const [treeData, setTreeData] = useState([]); // 树结构数据
  const [currentStation, setCurrentStation] = useState({}); // 当前选中站点
  const [searchWord, setSearchWord] = useState(''); // 关键字
  const [visibleParams, setvisibleParams] = useState({
    modalVisible: false, // 新增弹窗
    delVisible: false, // 删除弹窗
    editVisible: false, // 修改弹窗
    spinLoading: false, // 加载弹窗
    btnLoading: false,
    loading: false,
    checkBoxLoading: false,
  });
  const [flag, setFlag] = useState(1);
  const [dataList, setdataList] = useState([]);
  const [mulu, setMulu] = useState(true);
  const [optionsList, setOptionsList] = useState([]);
  // 侧边栏站点
  useEffect(() => {
    handleShowModal('spinLoading', true);
    getWebModuleTree({
      userMode: userMode || 'super',
      select: '',
      _version: 9999,
      _dc: Date.now(),
      node: -2,
    })
      .then(res => {
        handleShowModal('spinLoading', false);
        let arr = [];
        if (res) {
          arr.push(res.find(item => item.id === 'Web4StationRoot'));
        }
        if (arr[0]) {
          const stations = arr[0].children;
          setTreeData(stations);
          if (!currentStation.stationID) {
            setCurrentStation(stations[0]);
          } else {
            setCurrentStation(
              stations.find(s => s.stationID === currentStation.stationID) ||
                stations[0],
            );
          }
        }
      })
      .catch(err => {
        handleShowModal('spinLoading', false);
        console.error(err);
      });
  }, [flag]);
  // 右边对应的机构
  useEffect(() => {
    if (!currentStation.stationID) return;
    handleShowModal('loading', true);
    getAllGroup({ stationId: currentStation.stationID }).then(res => {
      if (res.code === 0) {
        let { data } = res;
        // for(let i=0;i<2000;i++){
        //   data.push({
        //     GroupId:i+10000,
        //     GroupName:`测试${i}`
        //   })
        // }
        handleShowModal('loading', false);
        setdataList(data);
        setOptionsList(data);
      }
    });
  }, [currentStation]);

  // 弹出模态框
  const handleShowModal = (key, value) => {
    setvisibleParams({ ...visibleParams, [key]: value });
  };
  // 获取搜索框的值
  const handleSearch = value => {
    if (value) {
      dataList.forEach(item => {
        item.color = item.GroupName.indexOf(value) > -1 ? '#f00' : '#333';
      });
      setdataList(lodash.cloneDeep(dataList));
      setSearchWord(value);
    }
  };
  // 搜索框点击进行锚点链接跳转
  const handleChange = value => {
    if (value) {
      let index = dataList.findIndex(item => item.GroupId === value);
      dataList.forEach(item => {
        item.color = '#333';
      });
      dataList[index].color = '#f00';
      setdataList(lodash.cloneDeep(dataList));
      let anchorElement = document.getElementById(`siteId${value}`);
      anchorElement.scrollIntoView();
    }
  };
  const confirmModal = e => {
    handleShowModal('modalVisible', false);
    setFlag(flag + 1);
  };
  const delModal = () => {
    handleShowModal('delVisible', false);
    setFlag(flag + 1);
  };
  const editModal = () => {
    handleShowModal('editVisible', false);
    setFlag(flag + 1);
  };
  const handleHide = () => {
    setMulu(!mulu);
  };
  const renderListItem = items =>
    items.map(t => (
      <List.Item
        onClick={() => {
          setCurrentStation(t);
        }}
        key={t.id}
        className={classnames({
          [styles.listItem]: true,
          [styles.selected]: currentStation.id === t.id,
        })}
      >
        {t.text}
      </List.Item>
    ));
  const handleChangeCollpase = (groupId, isShow) => {
    let index = dataList.findIndex(item => item.GroupId == groupId);
    dataList[index].isShow = !isShow;
    if (dataList[index].children && dataList[index].children.length > 0) {
      setdataList(lodash.cloneDeep(dataList));
      return;
    }
    handleShowModal('loading', true);
    getStationUserList({ stationID: currentStation.stationID, groupId }).then(
      res => {
        if (res.code === 0 && res.data) {
          handleShowModal('loading', false);
          dataList[index].children = res.data;
          setdataList(lodash.cloneDeep(dataList));
        }
      },
    );
  };
  // 每组全选全不选
  const handleChangeAll = (e, index) => {
    dataList[index].isChecked = e.target.checked;
    dataList[index].selectList = e.target.checked
      ? dataList[index].userList
      : [];
    dataList[index].children &&
      dataList[index].children.forEach(item => {
        item.isChecked = e.target.checked;
      });
    setdataList(lodash.cloneDeep(dataList));
  };
  // 单个选择checkbox
  const handleChangeSignel = (e, index, vIndex) => {
    dataList[index].children[vIndex].isChecked = e.target.checked;
    let checked =
      dataList[index].children.filter(item => item.isChecked).length ==
      dataList[index].children.length;
    dataList[index].isChecked = checked;
    let selectArray = dataList[index].children.filter(item => item.isChecked);
    dataList[index].selectList = selectArray;
    setdataList(lodash.cloneDeep(dataList));
  };
  // 提交
  const handleCommitBtn = () => {
    handleShowModal('btnLoading', true);
    let result = [];
    // 数据处理成后台需要的格式
    dataList.forEach(item => {
      if (item.selectList && item.selectList.length > 0) {
        let idArr = item.selectList.map(v => v.userID);
        item.isChecked && idArr.push(item.id);
        result.push(...idArr);
      }
    });
    chooseUserToStation(
      qs.stringify({
        userList: String(result.flat()),
        stationID: currentStation.stationID,
      }),
      {
        headers: {
          'content-type': 'application/x-www-form-urlencggoded;charset=UTF-8',
        },
      },
    )
      .then(res => {
        handleShowModal('btnLoading', false);

        if (res.success) {
          notification.success({
            message: '提示',
            duration: 3,
            description: '设置成功',
          });
        } else {
          notification.error({
            message: '提示',
            duration: 15,
            description: res.message,
          });
        }
      })
      .catch(err => {
        handleShowModal('btnLoading', false);
        console.log(err);
      });
  };
  return (
    <PageContainer>
      {/* <Row>
        <Col span={mulu ? 4 : 0}> */}
      <div className={styles.content}>
        <Card
          className={classnames({
            [styles.cardBox]: true,
            [styles.hideBox]: !mulu,
          })}
        >
          <Spin
            tip="loading...."
            spinning={visibleParams.spinLoading}
            style={{ marginTop: '20px' }}
          >
            <div className={userStyles.siteTitle}>
              <span>选择站点：</span>
            </div>
            <List>{renderListItem(treeData)}</List>
          </Spin>
          <AddModal
            visible={visibleParams.modalVisible}
            onCancel={() => handleShowModal('modalVisible', false)}
            confirmModal={confirmModal}
          />
          <DelModal
            visible={visibleParams.delVisible}
            stationId={currentStation.stationID}
            onCancel={() => handleShowModal('delVisible', false)}
            confirmModal={delModal}
          />
          <EditModal
            visible={visibleParams.editVisible}
            stationObj={currentStation}
            onCancel={() => handleShowModal('editVisible', false)}
            confirmModal={editModal}
          />
          <div>
            {mulu && (
              <Tooltip title="隐藏站点" className={styles.hide}>
                <DoubleLeftOutlined onClick={() => handleHide()} />
              </Tooltip>
            )}
            {!mulu && (
              <Tooltip title="显示站点" className={styles.hide}>
                <DoubleRightOutlined onClick={() => handleHide()} />
              </Tooltip>
            )}
          </div>
        </Card>
        {/* <div>
            {mulu && (
              <Tooltip title="隐藏站点" className={styles.hide}>
                <DoubleLeftOutlined onClick={() => handleHide()} />
              </Tooltip>
            )}
            {!mulu && (
              <Tooltip title="显示站点" className={styles.hide}>
                <DoubleRightOutlined onClick={() => handleHide()} />
              </Tooltip>
            )}
          </div> */}
        {/* </Col> */}
        {/* <Col span={mulu ? 0 : 1}>
          {mulu && (
            <Tooltip title="隐藏站点" className={styles.hide}>
              <DoubleLeftOutlined onClick={() => handleHide()} />
            </Tooltip>
          )}
          {!mulu && (
            <Tooltip title="显示站点" className={styles.hide}>
              <DoubleRightOutlined onClick={() => handleHide()} />
            </Tooltip>
          )}
        </Col> */}
        {/* <Col span={mulu ? 20 : 23}> */}
        <div
          className={classnames({
            [styles.boxR]: true,
            [styles.boxH]: mulu,
          })}
        >
          <Card
            className={classnames({
              [styles.cardBoxTop]: true,
              [styles.boxH]: mulu,
            })}
          >
            <Row align="middle">
              <Col span={1}>搜索</Col>
              <Col span={8}>
                <Select
                  style={{ width: '100%' }}
                  showSearch
                  placeholder={placeholder}
                  onSearch={handleSearch}
                  onSelect={handleChange}
                  filterOption={(input, option) =>
                    option.children.indexOf(input) >= 0
                  }
                >
                  {optionsList.map(item => (
                    <Option key={item.GroupId} value={item.GroupId}>
                      {item.GroupName}
                    </Option>
                  ))}
                </Select>
              </Col>
              {/* <Col span={3} /> */}
              <Col span={15} style={{ textAlign: 'right' }}>
                <Space size="small">
                  <Button
                    type="primary"
                    onClick={() => {
                      handleShowModal('modalVisible', true);
                    }}
                  >
                    新增
                  </Button>
                  <Button
                    type="primary"
                    onClick={() => {
                      handleShowModal('editVisible', true);
                    }}
                    disabled={
                      !currentStation.stationID || visibleParams.spinLoading
                    }
                  >
                    编辑
                  </Button>
                  <Button
                    danger
                    type="primary"
                    onClick={() => {
                      handleShowModal('delVisible', true);
                    }}
                    disabled={
                      !currentStation.stationID || visibleParams.spinLoading
                    }
                  >
                    删除
                  </Button>
                </Space>
              </Col>
            </Row>
          </Card>
          <Card
            className={classnames({
              [styles.boxH]: mulu,
              [styles.cardBoxR]: true,
            })}
          >
            {/* <Checkbox className={styles.siteAll}>全选/反选</Checkbox> */}
            <Spin spinning={visibleParams.loading}>
              {dataList.map((item, index) => (
                <Panels
                  {...item}
                  index={index}
                  key={item.GroupId}
                  handleChangeCollpase={handleChangeCollpase}
                  handleChangeAll={handleChangeAll}
                  handleChangeSignel={handleChangeSignel}
                />
              ))}
            </Spin>
            {dataList.length ? (
              <Button
                type="primary"
                className={styles.siteCommit}
                onClick={handleCommitBtn}
              >
                提交
              </Button>
            ) : null}
          </Card>
        </div>
      </div>
      {/* </Col>
      </Row> */}
    </PageContainer>
  );
};

const Panels = React.memo(props => {
  let { index, GroupId, GroupName, children, isChecked, isShow, color } = props;
  return (
    <div className={styles.sitePanel} key={GroupId} id={`siteId${GroupId}`}>
      <div
        className={styles.sitePanelHead}
        onClick={() => props.handleChangeCollpase(GroupId, isShow)}
      >
        {isShow ? (
          <UpOutlined className={styles.siteIcon} />
        ) : (
          <DownOutlined className={styles.siteIcon} />
        )}
        <p style={{ color }}>{GroupName}</p>
      </div>
      {isShow ? (
        <div className={styles.sitePanelCon}>
          <Checkbox
            key="0"
            className={styles.siteList}
            checked={isChecked}
            onClick={e => props.handleChangeAll(e, index)}
          >
            全选
          </Checkbox>
          {children &&
            children.map((v, vIndex) => (
              <CheckBoxRow
                {...v}
                index={index}
                vIndex={vIndex}
                key={v.userID}
                handleChangeSignel={props.handleChangeSignel}
              />
            ))}
        </div>
      ) : null}
    </div>
  );
});

const CheckBoxRow = React.memo(props => {
  let { vIndex, index, isChecked, userName } = props;
  return (
    <Checkbox
      className={styles.siteList}
      checked={isChecked}
      onClick={e => props.handleChangeSignel(e, index, vIndex)}
    >
      {userName}
    </Checkbox>
  );
});

export default React.memo(appConnector(SiteManage));
